<?php

declare(strict_types=1);

namespace Drupal\wsolution\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\wsolution\Entity\WsolutionUser;
use Drupal\wsolution\Entity\WsolutionWebservice;

use Symfony\Component\Yaml\Yaml;

/**
 * WSolution Webservice form.
 */
final class WsolutionWebserviceForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [WsolutionWebservice::class, 'load'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];
    $form['domain'] = [
      '#type' => 'url',
      '#title' => $this->t('Url domain'),
      '#default_value' => $this->entity->get('domain'),
      '#required' => TRUE,
    ];
    $default_value = NULL;
    $wsuid = $this->entity->get('wsuid');
    if (!empty($wsuid)) {
      $wsuser = $this->entityTypeManager->getStorage('wsolution_user')
        ->load($wsuid);
      if (!empty($wsuser) && $wsuser instanceof WsolutionUser) {
        $default_value = $wsuser;
      }
    }
    $form['wsuid'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Select a Wsolution User'),
      '#target_type' => 'wsolution_user',
      // Le machine name de votre entité de configuration
      '#description' => $this->t('Start typing the name of a Wsolution user.'),
      '#default_value' => $default_value,
    ];
    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $this->entity->get('description'),
    ];

    $form['tasks_yaml'] = [
      '#type' => 'textarea',

      '#required' => TRUE,
      '#title' => $this->t('Tasks Configuration (YAML)'),
      '#default_value' => Yaml::dump($this->entity->getTasks(), 4, 2),
      // Dump current tasks to YAML
      '#description' => $this->t('Enter the tasks configuration in YAML format.'),
      '#rows' => 20,
    ];
    return $form;
  }

  /**
   * @param array $element
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array
   */
  public function afterBuild(array $element, FormStateInterface $form_state) {
    if ($form_state->isProcessingInput()) {
      $taks = $form_state->getValue('tasks_yaml');
      if (!empty($taks)) {
        $tasks_data = Yaml::parse($taks);
        $form_state->setValue('tasks', $tasks_data);
      }
    }
    return parent::afterBuild($element, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Validate the YAML input.
    try {
      $tasks_data = Yaml::parse($form_state->getValue('tasks_yaml'));
      $form_state->setValue('tasks', $tasks_data);
    }
    catch (\Symfony\Component\Yaml\Exception\ParseException $e) {
      $form_state->setErrorByName('tasks_yaml', $this->t('Invalid YAML format: %message', ['%message' => $e->getMessage()]));
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match ($result) {
        \SAVED_NEW => $this->t('Created new example %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated example %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

}
