<?php

namespace Drupal\wsolution\PasseplatForms\Handler;

use Dakwamine\Component\ComponentBasedObject;
use Dakwamine\Component\Exception\UnmetDependencyException;
use PassePlat\Forms\Exception\WebServiceException;
use PassePlat\Forms\Vue\Response;
use PassePlat\Core\Exception\Exception;
use Drupal\wsolution_elastic\PasseplatElastic\Definitions\AnalyzableContentIndexDef;
use Drupal\wsolution_elastic\PasseplatElastic\Definitions\ErrorsIndexDef;
use PassePlat\Logger\Elastic\Exception\ElasticsearchException;
use Drupal\wsolution_elastic\PasseplatElastic\Service\DrupalElasticsearchService;
use PassePlat\Forms\Handler\WebServicesList;

class WebServicesHandler extends WebServicesList {

  public const string TEMP_STORE_KEY = 'webservices';
  use HandlerTrait;
  /**
   * Checks if the given web service has validation errors.
   *
   * @param string $webService
   *   The webservice ID.
   *
   * @return int
   *   The number of validation errors found.
   *
   * @throws UnmetDependencyException|ElasticsearchException
   */
  protected function checkIfHasErrorValidation(string $webService): int {
    $query = $this->buildQueryForCheck($webService);

    /** @var \PassePlat\Logger\Elastic\Service\ElasticsearchService $elasticSearchServer */
    $elasticSearchService = ComponentBasedObject::getRootComponentByClassName(DrupalElasticsearchService::class, TRUE);

    /** @var ErrorsIndexDef $errorIndex */
    $errorIndex = ComponentBasedObject::getRootComponentByClassName(ErrorsIndexDef::class, TRUE);

    $response = $elasticSearchService->search($errorIndex, $query);

    $errors = $response['aggregations']['total_errors']['value'];

    return !empty($errors) ? $errors : 0;
  }

  /**
   * Retrieves information from Elasticsearch for a given web service.
   *
   * @param string $webservice
   *   The web service ID.
   *
   * @return array
   *   Array containing the number of logs and last call date for the web
   *   service.
   *
   * @throws UnmetDependencyException|ElasticsearchException
   */
  private function getInformationFromES(string $webservice): array {
    $query = $this->buildQuery($webservice);

    /** @var \PassePlat\Logger\Elastic\Service\ElasticsearchService $elasticSearchServer */
    $elasticSearchService = ComponentBasedObject::getRootComponentByClassName(DrupalElasticsearchService::class, TRUE);

    /** @var AnalyzableContentIndexDef $analyzableIndex */
    $analyzableIndex = ComponentBasedObject::getRootComponentByClassName(AnalyzableContentIndexDef::class, TRUE);

    $response = $elasticSearchService->search($analyzableIndex, $query);

    return ($this->extractBucketMetrics($response['aggregations']));
  }

  /**
   * Retrieves the list of webservices.
   *
   * @return array
   *   The webservices list.
   *
   * @throws WebServiceException
   *   Thrown if there are any issues encountered at the PassePlat webservice
   *   level.
   */
  protected function getWebServices(): array {
    if (empty($this->webservices)) {
      $this->webservices = $this->getWebServicesList();
    }
    return $this->webservices;
  }

  public function handlePostRequest(): void {
    try {
      $result =    $this->getBlueprint('list');
      $this->setItems($result);
      $tempStoreFactory= \Drupal::service('tempstore.private');

      $tempstore = $tempStoreFactory->get('wsolution');

      $tempstore->set(self::TEMP_STORE_KEY, $result);
    //  Response::sendOk($result);
    }
    catch (\Exception $e) {
      Response::sendInternalServerError();
    }
  }
  /**
   * Sets the items to be displayed based on web service data.
   *
   * @param array $result
   *   Reference to the result array to populate.
   *
   * @throws Exception|UnmetDependencyException|WebServiceException
   */
  protected function setItems(array &$result) {
    $listWebServices = $this->listWebServices();

    foreach ($listWebServices as $webService) {
      $information = $this->getInformationFromES($webService);
      $result['data']['results'][] = [
        'results_item' => [
            'title' => $this->webservices[$webService]['name'] ?? 'undefined title',
            'domain' => $this->webservices[$webService]['domain'] ?? 'undefined domain',
            'url' => $webService,
            'nbOfLogs' => $information['nbOfLogs'] ?? 0,
            'lastCall' => $information['lastCall'] ?? 0,
            'edit' => '/admin/structure/wsolution/webservice-edit?wsid=' . $webService,
            'nbOfErrors' => $this->checkIfHasErrorValidation($webService),
            'errorsPage' => '/admin/structure/wsolution/errors?wsid=' . $webService,
            'dashboard' => '/admin/structure/wsolution/dashboard?wsid=' . $webService,
            'logsPage' => '/admin/structure/wsolution/logs?wsid=' . $webService,
            'copy' => $webService,
          ] + $this->checkWebServiceFeature($webService),
      ];
    }
  }

}
